Cartographer = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceModuleCore-2.0", "AceHook-2.1")
Cartographer.revision = tonumber(string.sub("$Revision: 21388 $", 12, -3))
Cartographer.version = "r" .. Cartographer.revision
Cartographer.date = string.sub("$Date: 2006-12-19 05:53:48 -0500 (Tue, 19 Dec 2006) $", 8, 17)

local Dewdrop = AceLibrary("Dewdrop-2.0")

local L = AceLibrary("AceLocale-2.2"):new("Cartographer")
local BZ = AceLibrary("Babble-Zone-2.2")
local Tablet = AceLibrary("Tablet-2.0")

BINDING_HEADER_CARTOGRAPHER = "Cartographer"
BINDING_NAME_CARTOGRAPHER_OPENALTERNATEMAP = "Open alternate map"

L:RegisterTranslations("enUS", function() return {
	["Active"] = "Enabled", -- remove in a while (2006-12-08)
	["Enabled"] = true,
	["Suspend/resume this module."] = true,
	
	["Right-Click on map to zoom out"] = true,
	["Left-Click on map to zoom in"] = true,
	
	["Go to %s"] = true,
	
	["Azeroth"] = true,
	["Cosmic map"] = true,
	
	["Open Cartographer menu"] = true,
	["Open location menu"] = true,
} end)

L:RegisterTranslations("deDE", function() return {
	["Enabled"] = "Aktiviert",
	["Suspend/resume this module."] = "Aktiviert/Deaktiviert dieses Modul",
	
	["Right-Click on map to zoom out"] = "Rechts-Klick auf Karte um raus zu zoomen",
	["Left-Click on map to zoom in"] = "Links-Klick um rein zu zoomen",
	
	["Go to %s"] = "Gehe zu %s",
	
	["Azeroth"] = "Azeroth",
	["Cosmic map"] = "Kosmische Karte",
	
	["Open Cartographer menu"] = "Cartographer Menü öffnen",
	["Open location menu"] = "Positions Menü öffnen",
} end)

L:RegisterTranslations("frFR", function() return {
	["Active"] = "Activé",
	["Enabled"] = "Activé",
	["Suspend/resume this module."] = "Active ou non ce module.",

	["Right-Click on map to zoom out"] = "Clic-droit sur la carte pour effectuer un zoom arrière",
	["Left-Click on map to zoom in"] = "Clic-gauche sur la carte pour effectuer un zoom avant",

	["Go to %s"] = "Allez à %s",

	--["Azeroth"] = true,
	["Cosmic map"] = "Carte cosmique",

	["Open Cartographer menu"] = "Ouvrir le menu Cartographer",
	["Open location menu"] = "Ouvrir le menu de situation",
} end)

L:RegisterTranslations("koKR", function() return {
	["Active"] = "활성화",
	["Enabled"] = "활성화",
	["Suspend/resume this module."] = "모듈을 켜거나 끔니다.",
	
	["Right-Click on map to zoom out"] = "오른쪽-클릭 : 축소",
	["Left-Click on map to zoom in"] = "왼쪽-클릭 : 확대",
	
	["Go to %s"] = "%s 바로가기",
	
	["Azeroth"] = "아제로스",
	["Cosmic map"] = "세계 지도",
	
	["Open Cartographer menu"] = "Cartographer 메뉴 열기",
	["Open location menu"] = "위치 메뉴 열기",
} end)

L:RegisterTranslations("zhCN", function() return {
--	["Enabled"] = true,
--	["Suspend/resume this module."] = true,
	
--	["Right-Click on map to zoom out"] = true,
--	["Left-Click on map to zoom in"] = true,
	
--	["Go to %s"] = true,
	
--	["Azeroth"] = true,
--	["Cosmic map"] = true,
	
--	["Open Cartographer menu"] = true,
--	["Open location menu"] = true,
} end)

L:RegisterTranslations("zhTW", function() return {
--	["Enabled"] = true,
--	["Suspend/resume this module."] = true,
	
--	["Right-Click on map to zoom out"] = true,
--	["Left-Click on map to zoom in"] = true,
	
--	["Go to %s"] = true,
	
--	["Azeroth"] = true,
--	["Cosmic map"] = true,
	
--	["Open Cartographer menu"] = true,
--	["Open location menu"] = true,
} end)

L:RegisterTranslations("esES", function() return {
	["Active"] = "Activo", -- remove in a while (2006-12-08)
	["Enabled"] = "Activado",
	["Suspend/resume this module."] = "Parar/Continuar este m\195\179dulo",
	
	["Right-Click on map to zoom out"] = "Clic-Derecho en el mapa para alejarte",
	["Left-Click on map to zoom in"] = "Clic-Izquierdo en el mapa para acercarte",

	["Go to %s"] = "Ir a %s",
	
	["Azeroth"] = "Azeroth",
	["Cosmic map"] = "Mapa c\195\179smico",
	
	["Open Cartographer menu"] = "Abrir el men\195\186 de Cartographer",
	["Open location menu"] = "Abrir el men\195\186 de lugares",
} end)

Cartographer:RegisterDB("CartographerDB")

Cartographer.options = {
	type = 'group',
	args = {
	},
}
Dewdrop:InjectAceOptionsTable(Cartographer, Cartographer.options)
Cartographer.options.args.standby = nil

function Cartographer:OnInitialize()
	self.gotoOptions = {
		type = 'group',
		args = {
			player = {
				name = string.format(L["Go to %s"], GetRealZoneText()),
				desc = string.format(L["Go to %s"], GetRealZoneText()),
				type = 'execute',
				func = function()
					SetMapToCurrentZone()
				end,
				order = 1
			}
		},
	}
	
	for i,continent in ipairs { GetMapContinents() } do
		local i = i
		local validate = { GetMapZones(i) }
		self.gotoOptions.args[string.gsub(continent, " ", "-")] = {
			name = continent,
			desc = continent,
			type = 'text',
			validate = validate,
			get = function() return self:GetCurrentInstance() or GetCurrentMapContinent() == i and validate[GetCurrentMapZone()] end,
			set = function(text)
				for j,v in ipairs(validate) do
					if v == text then
						SetMapZoom(i, j)
						break
					end
				end
			end,
		}
	end
end

local magnifyingGlassTexts = { L["Right-Click on map to zoom out"], L["Left-Click on map to zoom in"] }
local currentLeftSide, currentRightSide
function Cartographer:OnEnable()
	WorldMapContinentDropDown:Hide()
	WorldMapZoneDropDown:Hide()
	
	local CartographerOptionsButton = CreateFrame("Button", "CartographerOptionsButton", WorldMapFrame, "UIPanelButtonTemplate")
	CartographerOptionsButton:SetText("Cartographer")
	local width = CartographerOptionsButton:GetTextWidth() + 30
	if width < 110 then
		width = 110
	end
	CartographerOptionsButton:SetWidth(width)
	CartographerOptionsButton:SetHeight(22)
	WorldMapZoomOutButton:Hide()
	CartographerOptionsButton:SetScript("OnClick", function()
		Dewdrop:Register(this,
			'children', self.options,
			'dontHook', true,
			'point', "TOPRIGHT",
			'relativePoint', "BOTTOMRIGHT"
		)
		this:SetScript("OnClick", function()
			if Dewdrop:IsOpen(this) then
				Dewdrop:Close()
			else
				Dewdrop:Open(this)
			end
		end)
		this:GetScript("OnClick")()
	end)
	CartographerOptionsButton:SetScript("OnEnter", function(this)
		GameTooltip_SetDefaultAnchor(GameTooltip, this)
		GameTooltip:SetText(L["Open Cartographer menu"], HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
		GameTooltip:Show()
	end)
	CartographerOptionsButton:SetScript("OnLeave", function(this)
		GameTooltip:Hide()
	end)
	self:AddMapButton(CartographerOptionsButton, 1)
	
	local CartographerGoToButton = CreateFrame("Button", "CartographerGoToButton", WorldMapFrame, "UIPanelButtonTemplate")
	CartographerGoToButton:SetText(UNKNOWN)
	local width = CartographerGoToButton:GetTextWidth() + 30
	if width < 220 then
		width = 220
	end
	CartographerGoToButton:SetWidth(width)
	CartographerGoToButton:SetHeight(22)
	CartographerGoToButton:SetScript("OnClick", function()
		Dewdrop:Register(this,
			'children', self.gotoOptions,
			'dontHook', true,
			'point', "TOPRIGHT",
			'relativePoint', "BOTTOMRIGHT"
		)
		this:SetScript("OnClick", function()
			if Dewdrop:IsOpen(this) then
				Dewdrop:Close()
			else
				Dewdrop:Open(this)
			end
		end)
		this:GetScript("OnClick")()
	end)
	CartographerGoToButton:SetScript("OnEnter", function(this)
		GameTooltip_SetDefaultAnchor(GameTooltip, this)
		GameTooltip:SetText(L["Open location menu"], HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
		GameTooltip:Show()
	end)
	CartographerGoToButton:SetScript("OnLeave", function(this)
		GameTooltip:Hide()
	end)
	self:AddMapButton(CartographerGoToButton, 0)
	
	self:Hook(WorldMapFrame, "Hide", "WorldMapFrame_Hide", true)
	self:Hook(WorldMapFrame, "Show", "WorldMapFrame_Show", true)
	self:RegisterEvent("Cartographer_MapOpened")
	self:RegisterEvent("Cartographer_MapClosed")
	self:RegisterEvent("Cartographer_ChangeZone")
	self:RegisterEvent("WORLD_MAP_UPDATE")
	self:Cartographer_ChangeZone(self:GetCurrentEnglishZoneName(), self:GetCurrentLocalizedZoneName())
	
	WorldMapMagnifyingGlassButton:SetText(table.concat(magnifyingGlassTexts, "\n"))
	
	if (not GetBindingAction("ALT-M") or GetBindingAction("ALT-M") == "") and not GetBindingKey("CARTOGRAPHER_OPENALTERNATEMAP") then
		SetBinding("ALT-M", "CARTOGRAPHER_OPENALTERNATEMAP")
	end
	
	for i = 1, 2 do
		local side = i == 1 and "LEFT" or "RIGHT"
		local s = "Cartographer_" .. side .. "Side"
		Tablet:Register(s,
			'data', {},
			'children', side == "LEFT" and function()
				if currentLeftSide then
					currentLeftSide:OnCartographerLeftTabletRequest()
				end
			end or function()
				if currentRightSide then
					currentRightSide:OnCartographerRightTabletRequest()
				end
			end,
			'clickable', true,
			'dontHook', true,
			'cantAttach', true,
			'frameLevel', 11,
			'movable', false,
			'hideWhenEmpty', true,
			'parent', WorldMapFrame,
			'showTitleWhenDetached', true,
			'showHintWhenDetached', true,
			'positionFunc', side == "LEFT" and function(this)
				this:SetPoint("TOPLEFT", WorldMapDetailFrame, "TOPLEFT")
				this:SetPoint("BOTTOMLEFT", WorldMapDetailFrame, "BOTTOMLEFT")
			end or function(this)
				this:SetPoint("TOPRIGHT", WorldMapDetailFrame, "TOPRIGHT")
				this:SetPoint("BOTTOMRIGHT", WorldMapDetailFrame, "BOTTOMRIGHT")
			end
		)
		if WorldMapFrame:IsShown() then
			Tablet:Open(s)
		end
	end
end

function Cartographer:WorldMapFrame_Hide(this)
	local shown = WorldMapFrame:IsShown()
	self.hooks[this].Hide(this)
	if shown then
		self:TriggerEvent("Cartographer_MapClosed")
	end
end

function Cartographer:WorldMapFrame_Show(this)
	local shown = WorldMapFrame:IsShown()
	self.hooks[this].Show(this)
	if not shown then
		self:TriggerEvent("Cartographer_MapOpened")
	end
end

function Cartographer:WORLD_MAP_UPDATE()
	self:TriggerEvent("Cartographer_ChangeZone", self:GetCurrentEnglishZoneName(), self:GetCurrentLocalizedZoneName())
end

local function func(self)
	if currentLeftSide then
		Tablet:Open("Cartographer_LEFTSide")
--		Tablet:Refresh("Cartographer_LEFTSide")
	end
	if currentRightSide then
		Tablet:Open("Cartographer_RIGHTSide")
	end
end

function Cartographer:Cartographer_MapOpened()
	self:ScheduleEvent(func, 0, self)
end

function Cartographer:Cartographer_MapClosed()
	if Dewdrop:IsOpen(CartographerGoToButton) or Dewdrop:IsOpen(CartographerOptionsButton) then
		Dewdrop:Close()
	end
	SetMapToCurrentZone()
end

local continents = { GetMapContinents() }
continents[0] = L["Azeroth"]
continents[-1] = L["Cosmic map"]

function Cartographer:Cartographer_ChangeZone(zone, localZone)
	if localZone then
		CartographerGoToButton:SetText(localZone)
		Cartographer.gotoOptions.args.player.name = string.format(L["Go to %s"], GetRealZoneText())
		Cartographer.gotoOptions.args.player.desc = Cartographer.gotoOptions.args.player.name
	else
		CartographerGoToButton:SetText(continents[GetCurrentMapContinent()] or UNKNOWN)
	end
end

function Cartographer:OnProfileEnable(alpha, bravo)
	for _,module in self:IterateModules() do
		if self:IsModuleActive(module) and type(module.OnProfileEnable) == "function" then
			module:OnProfileEnable(alpha, bravo)
		end
	end
end

function Cartographer:AddToMagnifyingGlass(text)
	for _,v in ipairs(magnifyingGlassTexts) do
		if v == text then
			error(string.format("Cannot add %q to magnifying glass, it already exists", text), 2)
		end
	end
	table.insert(magnifyingGlassTexts, text)
	WorldMapMagnifyingGlassButton:SetText(table.concat(magnifyingGlassTexts, "\n"))
end

function Cartographer:RemoveFromMagnifyingGlass(text)
	local id
	for i,v in ipairs(magnifyingGlassTexts) do
		if v == text then
			id = i
			break
		end
	end
	if not id then
		error(string.format("Cannot remove %q from magnifying glass, it does not exist", text), 2)
	end
	table.remove(magnifyingGlassTexts, id)
	
	WorldMapMagnifyingGlassButton:SetText(table.concat(magnifyingGlassTexts, "\n"))
end

local mapButtons = {}

local function my_sort(alpha, bravo)
	if not alpha or not bravo then
		return false
	end
	return mapButtons[alpha] < mapButtons[bravo]
end

local tmp = {}
local function refreshMapButtons()
	for k in pairs(mapButtons) do
		table.insert(tmp, k)
	end
	
	table.sort(tmp, my_sort)
	
	local width = -10
	for _,frame in ipairs(tmp) do
		width = width + frame:GetWidth() + 10
	end
	
	local last = tmp[1]
	last:SetPoint("BOTTOM", WorldMapZoomOutButton, "BOTTOM", 0, 0)
	last:SetPoint("LEFT", WorldMapDetailFrame, "CENTER", -width/2, 0)
	for i = 2, #tmp do
		local this = tmp[i]
		this:SetPoint("LEFT", last, "RIGHT", 10, 0)
		last = this
	end
	
	for k in pairs(tmp) do
		tmp[k] = nil
	end
end

function Cartographer:AddMapButton(frame, order)
	if mapButtons[frame] then
		error(string.format("Cannot add %q to map buttons, it already exists", frame:GetName() or "anonymous frame"), 2)
	end
	mapButtons[frame] = order
	
	refreshMapButtons()
end

function Cartographer:RemoveMapButton(frame)
	if not mapButtons[frame] then
		error(string.format("Cannot remove %q from map buttons, it does not exist", frame:GetName() or "anonymous frame"), 2)
	end
	mapButtons[frame] = nil
	
	refreshMapButtons()
end

local currentInstance
function Cartographer:SetCurrentInstance(zone)
	if currentInstance ~= zone then
		currentInstance = zone
		self:TriggerEvent("Cartographer_SetCurrentInstance", zone)
		self:TriggerEvent("Cartographer_ChangeZone", self:GetCurrentEnglishZoneName(), self:GetCurrentLocalizedZoneName())
	end
end

function Cartographer:GetCurrentInstance()
	return currentInstance
end

local instanceWorldMapButton
function Cartographer:RegisterInstanceWorldMapButton(frame)
	instanceWorldMapButton = frame
	self:TriggerEvent("Cartographer_RegisterInstanceWorldMapButton", frame)
end

function Cartographer:GetInstanceWorldMapButton()
	return instanceWorldMapButton
end

local mapZones = setmetatable({}, {__index = function(self, key)
	self[key] = { GetMapZones(key) }
	return self[key]
end })

function Cartographer:GetCurrentLocalizedZoneName()
	if currentInstance then
		return BZ[currentInstance]
	end
	local z = mapZones[GetCurrentMapContinent()][GetCurrentMapZone()]
	if z then
		return z
	else
		local map = GetMapInfo()
		if map == "WarsongGulch" then
			return BZ["Warsong Gulch"]
		elseif map == "ArathiBasin" then
			return BZ["Arathi Basin"]
		elseif map == "AlteracValley" then
			return BZ["Alterac Valley"]
		elseif map == "NetherstormArena" then
			return BZ["Eye of the Storm"]
		end
		return nil
	end
end

function Cartographer:GetCurrentEnglishZoneName()
	if currentInstance then
		return currentInstance
	end
	local z = mapZones[GetCurrentMapContinent()][GetCurrentMapZone()]
	if z then
		return BZ:GetReverseTranslation(z)
	else
		local map = GetMapInfo()
		if map == "WarsongGulch" then
			return "Warsong Gulch"
		elseif map == "ArathiBasin" then
			return "Arathi Basin"
		elseif map == "AlteracValley" then
			return "Alterac Valley"
		elseif map == "NetherstormArena" then
			return "Eye of the Storm"
		end
		return nil
	end
end

function Cartographer:OpenAlternateMap()
	if self:GetProfile() == "Default" or self:GetProfile() == "Alternate" then
		self:SetProfile(self:GetProfile() == "Default" and "Alternate" or "Default")
	end
	
	if not WorldMapFrame:IsShown() then
		ToggleWorldMap()
	end
end

function Cartographer:AcquireSideTablet(side, id)
	if side ~= "LEFT" and side ~= "RIGHT" then
		error('Argument #2 to `AcquireSideTablet\' must either be "LEFT" or "RIGHT".', 2)
	end
	AceLibrary.argCheck(self, id, 3, "table")
	if side == "LEFT" then
		if type(id.OnCartographerLeftTabletRequest) ~= "function" then
			error("Argument #3 must include the method `OnCartographerLeftTabletRequest'.", 2)
		end
		currentLeftSide = id
	else -- RIGHT
		if type(id.OnCartographerRightTabletRequest) ~= "function" then
			error("Argument #3 must include the method `OnCartographerRightTabletRequest'.", 2)
		end
		currentRightSide = id
	end
	local s = "Cartographer_" .. side .. "Side"
	if WorldMapFrame:IsShown() then
		Tablet:Refresh(s)
	end
end

function Cartographer:ReleaseSideTablet(side, id)
	if side ~= "LEFT" and side ~= "RIGHT" then
		error('Argument #2 to `AcquireSideTablet\' must either be "LEFT" or "RIGHT".', 2)
	end
	AceLibrary.argCheck(self, id, 3, "table")
	if side == "LEFT" then
		if id == currentLeftSide then
			currentLeftSide = nil
		else
			return
		end
	else -- RIGHT
		if id == currentRightSide then
			currentRightSide = nil
		else
			return
		end
	end
	Tablet:Refresh("Cartographer_" .. side .. "Side")
end

function Cartographer:GetCurrentSideTabletHandler(side)
	if side ~= "LEFT" and side ~= "RIGHT" then
		error('Argument #2 to `AcquireSideTablet\' must either be "LEFT" or "RIGHT".', 2)
	end
	if side == "LEFT" then
		return currentLeftSide
	else -- RIGHT
		return currentRightSide
	end
end

function Cartographer:RefreshSideTablet(side, id)
	if not side then
		self:RefreshSideTablet("LEFT", id)
		self:RefreshSideTablet("RIGHT", id)
		return
	end
	if side ~= "LEFT" and side ~= "RIGHT" then
		error('Argument #2 to `AcquireSideTablet\' must either be "LEFT" or "RIGHT".', 2)
	end
	AceLibrary.argCheck(self, id, 3, "table", "nil")
	local s = "Cartographer_" .. side .. "Side"
	--if not Tablet:IsRegistered(s) then
	--	return
	--end
	if side == "LEFT" then
		if id and id ~= currentLeftSide then
			return
		end
	else -- RIGHT
		if id and id ~= currentRightSide then
			return
		end
	end
	Tablet:Open(s)
end
