﻿local revision = tonumber(string.sub("$Revision: 21244 $", 12, -3))
if revision > Cartographer.revision then
	Cartographer.version = "r" .. revision
	Cartographer.revision = revision
	Cartographer.date = string.sub("$Date: 2006-12-18 11:55:27 -0500 (Mon, 18 Dec 2006) $", 8, 17)
end

local L = AceLibrary("AceLocale-2.2"):new("Cartographer-InstanceMaps")

L:RegisterTranslations("enUS", function() return {
	["Instance Maps"] = true,
	["Module which provides maps of instances."] = true,
	
	["Instances"] = true,
} end)

L:RegisterTranslations("deDE", function() return {
	["Instance Maps"] = "Instanz Karten",
	["Module which provides maps of instances."] = "Modul das Instanz Karten zur Verfügung stellt.",
} end)

L:RegisterTranslations("frFR", function() return {
	["Instance Maps"] = "Cartes d'instance",
	["Module which provides maps of instances."] = "Module fournissant des cartes pour les instances.",

	--["Instances"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Instance Maps"] = "인스턴스 던전",
	["Module which provides maps of instances."] = "인스턴스 던전을 보여줍니다.",
	
	["Instances"] = "인스턴스",
} end)

L:RegisterTranslations("zhTW", function() return {
--	["Instance Maps"] = true,
--	["Module which provides maps of instances."] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
--	["Instance Maps"] = true,
--	["Module which provides maps of instances."] = true,
} end)

L:RegisterTranslations("esES", function() return {
	["Instance Maps"] = "Mapas de Instancia",
	["Module which provides maps of instances."] = "M\195\179dulo que provee mapas de instancias",
	
	["Instances"] = "Instancias",
} end)

Cartographer_InstanceMaps = Cartographer:NewModule("InstanceMaps", "AceHook-2.1")

local BZ = AceLibrary("Babble-Zone-2.2")
local Tourist = AceLibrary("Tourist-2.0")

local instanceToTexture = {
	["Ahn'Qiraj"] = "TempleofAhnQiraj",
	["Blackfathom Deeps"] = "BlackfathomDeeps",
	["Blackrock Depths"] = "BlackrockDepths",
	["Blackwing Lair"] = "BlackwingLair",
	["Dire Maul"] = "DireMaul",
	["Dire Maul (East)"] = "DireMaulEast",
	["Dire Maul (North)"] = "DireMaulNorth",
	["Dire Maul (West)"] = "DireMaulWest",
	["Gnomeregan"] = "Gnomeregan",
	["Hellfire Ramparts"] = "HCRamparts",
	["Lower Blackrock Spire"] = "BlackrockSpireLower",
	["Maraudon"] = "Maraudon",
	["Molten Core"] = "MoltenCore",
	["Naxxramas"] = "Naxxramas",
	["Onyxia's Lair"] = "OnyxiasLair",
	["Ragefire Chasm"] = "RagefireChasm",
	["Razorfen Downs"] = "RazorfenDowns",
	["Razorfen Kraul"] = "RazorfenKraul",
	["Ruins of Ahn'Qiraj"] = "RuinsofAhnQiraj",
	["Scarlet Monastery"] = "ScarletMonastery",
	["Scholomance"] = "Scholomance",
	["Shadowfang Keep"] = "ShadowfangKeep",
	["Stratholme"] = "Stratholme",
	["The Blood Furnace"] = "HCBloodFurnaces",
	["The Deadmines"] = "TheDeadmines",
	["The Shattered Halls"] = "HCShatteredHalls",
	["The Slave Pens"] = "TheSlavePens",
	["The Steamvault"] = "TheSteamvault",
	["The Stockade"] = "TheStockades",
	["The Underbog"] = "TheUnderbog",
	["The Temple of Atal'Hakkar"] = "TheSunkenTemple",
	["Uldaman"] = "Uldaman",
	["Upper Blackrock Spire"] = "BlackrockSpireUpper",
	["Wailing Caverns"] = "WailingCaverns",
	["Zul'Farrak"] = "ZulFarrak",
	["Zul'Gurub"] = "ZulGurub",
}

function Cartographer_InstanceMaps:OnInitialize()
	self.name = L["Instance Maps"]
	self.title = L["Instance Maps"]
	Cartographer.options.args.InstanceMaps = {
		name = L["Instance Maps"],
		desc = L["Module which provides maps of instances."],
		type = 'group',
		args = {
			toggle = {
				name = AceLibrary("AceLocale-2.2"):new("Cartographer")["Enabled"],
				desc = AceLibrary("AceLocale-2.2"):new("Cartographer")["Suspend/resume this module."],
				type = "toggle",
				order = -1,
				get = function() return Cartographer:IsModuleActive(self) end,
				set = function() Cartographer:ToggleModuleActive(self) end
			},
		},
		handler = self,
	}
	
	local validate = {}
	for k,v in pairs(instanceToTexture) do
		validate[k] = BZ[k]
	end
	
	Cartographer.gotoOptions.args.Instances = {
		name = L["Instances"],
		desc = L["Instances"],
		type = 'text',
		get = function()
			return Cartographer:GetCurrentInstance()
		end,
		set = "ShowInstance",
		validate = validate,
		handler = self,
		hidden = function() return not Cartographer:IsModuleActive(self) end
	}
end

local CartographerInstanceFrame
function Cartographer_InstanceMaps:OnEnable()
	self:SecureHook("SetMapZoom")
	self:Hook("SetMapToCurrentZone", true)
end

function Cartographer_InstanceMaps:OnDisable()
	self:HideInstanceFrame()
end

function Cartographer_InstanceMaps:SetMapToCurrentZone()
	local zoneText = GetRealZoneText()
	if zoneText == BZ["Blackrock Spire"] then
		zoneText = BZ["Upper Blackrock Spire"]
	end
	if IsInInstance() and BZ:HasReverseTranslation(zoneText) and instanceToTexture[BZ:GetReverseTranslation(zoneText)] then
		self:ShowInstance(BZ:GetReverseTranslation(zoneText))
	else
		self:HideInstanceFrame()
		return self.hooks.SetMapToCurrentZone()
	end
end

function Cartographer_InstanceMaps:ShowInstanceFrame()
	if not CartographerInstanceFrame then
		CartographerInstanceFrame = CreateFrame("Button", "CartographerInstanceFrame", WorldMapFrame)
		CartographerInstanceFrame:SetPoint("CENTER", WorldMapButton, "CENTER")
		CartographerInstanceFrame:SetWidth(WorldMapButton:GetWidth())
		CartographerInstanceFrame:SetHeight(WorldMapButton:GetHeight())
		CartographerInstanceFrame:Hide()
		CartographerInstanceFrame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
		CartographerInstanceFrame:SetScript("OnClick", function()
			if arg1 == "RightButton" then
				local zone = BZ[Cartographer:GetCurrentInstance()]
				if Tourist:IsInKalimdor(zone) then
					SetMapZoom(1)
				elseif Tourist:IsInEasternKingdoms(zone) then
					SetMapZoom(2)
				elseif Tourist:IsInOutland(zone) then
					SetMapZoom(3)
				else
					SetMapZoom(0)
				end
			end
		end)
		local bgf = CreateFrame("Frame", nil, WorldMapFrame)
		local bg = bgf:CreateTexture(nil, "BACKGROUND")
		CartographerInstanceFrame.backgroundframe = bgf
		bgf:Hide()
		CartographerInstanceFrame.background = bg
		bg:SetTexture(0,0,0)
		bg:SetPoint("CENTER", CartographerInstanceFrame, "CENTER")
		bg:SetWidth(CartographerInstanceFrame:GetWidth())
		bg:SetHeight(CartographerInstanceFrame:GetHeight())
		local texture = CartographerInstanceFrame:CreateTexture(nil, "ARTWORK")
		CartographerInstanceFrame.texture = texture
		texture:SetPoint("CENTER", CartographerInstanceFrame, "CENTER")
		Cartographer:RegisterInstanceWorldMapButton(CartographerInstanceFrame)
	end
	if CartographerInstanceFrame:IsShown() then
		return
	end
	CartographerInstanceFrame:Show()
	CartographerInstanceFrame.backgroundframe:Show()
	WorldMapDetailFrame:Hide()
	WorldMapButton:Hide()
	ShowWorldMapArrowFrame(nil)
end

function Cartographer_InstanceMaps:HideInstanceFrame()
	Cartographer:SetCurrentInstance(nil)
	if not CartographerInstanceFrame or not CartographerInstanceFrame:IsShown() then
		return
	end
	CartographerInstanceFrame:Hide()
	CartographerInstanceFrame.backgroundframe:Hide()
	WorldMapDetailFrame:Show()
	WorldMapButton:Show()
	ShowWorldMapArrowFrame(1)
end

function Cartographer_InstanceMaps:ShowInstance(realZone)
	if not instanceToTexture[realZone] then
		return
	end
	local id = this and this:GetID() or 0
	local zone = BZ[realZone]
	if Tourist:IsInKalimdor(zone) then
		SetMapZoom(1, 1)
	elseif Tourist:IsInEasternKingdoms(zone) then
		SetMapZoom(2, 1)
	elseif Tourist:IsInOutland(zone) then
		SetMapZoom(3, 1)
	else
		SetMapZoom(0)
	end
	Cartographer_InstanceMaps:ShowInstanceFrame()
	UIDropDownMenu_SetSelectedID(WorldMapContinentDropDown, self.instances_id)
	UIDropDownMenu_SetText(L["Instances"], WorldMapContinentDropDown)
	UIDropDownMenu_SetSelectedID(WorldMapZoneDropDown, id)
	UIDropDownMenu_SetText(zone, WorldMapZoneDropDown)
	Cartographer:SetCurrentInstance(realZone)
	CartographerInstanceFrame.texture:SetWidth(0)
	CartographerInstanceFrame.texture:SetHeight(0)
	CartographerInstanceFrame.texture:SetTexture("Interface\\AddOns\\Cartographer\\Instances\\" .. instanceToTexture[realZone])
	local currWidth, currHeight = CartographerInstanceFrame.texture:GetWidth(), CartographerInstanceFrame.texture:GetHeight()
	if currWidth == 0 then
		currWidth, currHeight = 512, 512
	end
	local newHeight = CartographerInstanceFrame:GetHeight()/CartographerInstanceFrame:GetScale() * 0.95
	local newWidth = currWidth*newHeight/currHeight
	CartographerInstanceFrame.texture:SetWidth(newWidth)
	CartographerInstanceFrame.texture:SetHeight(newHeight)
end

function Cartographer_InstanceMaps:SetMapZoom()
	self:HideInstanceFrame()
	if GetCurrentMapZone() == 0 then
		UIDropDownMenu_SetText("", WorldMapZoneDropDown)
	end
end